/* ab_logginc.cc
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ab_logging.hh"
#include <cstdio>
#include <cstdarg>
#include <list>

enum ab_logging_level ab_log_level = LOG_CRITICAL;

static std::list< std::string > file_filters;
static std::list< std::string > function_filters;

void ab_log( enum ab_logging_level level, const char *file, const char *function, int line, const char *message, ... )
{
    if ( ab_log_level < level ) return;

    bool match = false;
    bool found = false;

    if ( ! file_filters.empty() ) {
        match = true;
        for ( auto &ff : file_filters ) {
            if ( std::string( file ).find( ff ) != std::string::npos ) {
                found = true;
                break;
            }
        }
    }
    
    if ( ! function_filters.empty() ) {
        match = true;
        for ( auto &ff : function_filters ) {
            if ( std::string( function ).find( ff ) != std::string::npos ) {
                found = true;
                break;
            }
        }
    }

    if ( match && ! found ) {
        return;
    }

    va_list args;
    va_start( args, message );

    switch ( level ) {
        case LOG_CRITICAL:
            printf( "[CRITICAL] " );
            break;
        case LOG_VERBOSE:
            printf( "[VERBOSE ] " );
            break;
        case LOG_DEBUG:
            printf( "[DEBUG   ] " );
            break;
        default:
            break;
    }

    vprintf( message, args );
    va_end( args );

    printf( " [%s:%s:%d]", file, function, line );
    printf( "\n" );
}

void ab_set_log_level( enum ab_logging_level l )
{
    ab_log_level = l;
}

void ab_add_file_filter( const std::string &filename_infix )
{
    file_filters.push_back( filename_infix );
}

void ab_add_function_filter( const std::string &function_infix )
{
    function_filters.push_back( function_infix );
}
