/* ab_logginc.hh
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef AB_LOGGING_HH
#define AB_LOGGING_HH

#include <string>

enum ab_logging_level {
                       LOG_NOTHING,
                       LOG_CRITICAL,
                       LOG_VERBOSE,
                       LOG_DEBUG
};

extern enum ab_logging_level ab_log_level;

void ab_set_log_level( enum ab_logging_level l );
void ab_add_file_filter( const std::string &filename_infix );
void ab_add_function_filter( const std::string &function_infix );
void ab_log( enum ab_logging_level level, const char *file, const char *function, int line, const char *message, ... );

#define LOG_DEBUG(message...) ab_log( LOG_DEBUG, __FILE__, __FUNCTION__, __LINE__, message )
#define LOG_VERBOSE(message...) ab_log( LOG_VERBOSE, __FILE__, __FUNCTION__, __LINE__, message )
#define LOG_CRITICAL(message...) ab_log( LOG_CRITICAL, __FILE__, __FUNCTION__, __LINE__, message )

#endif /* AB_LOGGING_HH */
