/* chunk_anhd.hh
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CHUNK_ANHD_HH
#define CHUNK_ANHD_HH

#include "chunk_generic.hh"

class ChunkANHD : public ChunkGeneric {
public:
    json_t *dump_as_json( const shared_properties_t &shared_props,
                          bool include_raw ) const override;

    static const uint8_t OPERATION_DIRECT = 0;
    static const uint8_t OPERATION_XOR = 1;
    static const uint8_t OPERATION_LONG_DELTA = 2;
    static const uint8_t OPERATION_SHORT_DELTA = 3;
    static const uint8_t OPERATION_SHORT_LONG_DELTA = 4;
    static const uint8_t OPERATION_VERTICAL_DELTA = 5;
    static const uint8_t OPERATION_SHORT_LONG_VDELTA = 7;
    static const uint8_t OPERATION_DELTA8 = 8;

    uint8_t operation() const;
    uint8_t mask() const;
    uint16_t w() const;
    uint16_t h() const;
    int16_t x() const;
    int16_t y() const;
    uint32_t abstime() const;
    uint32_t reltime() const;
    uint8_t interleave() const;
    uint32_t bits() const;
protected:
    int parse_chunk( const std::vector< char > &data ) override;
private:
    uint8_t m_operation;
    uint8_t m_mask;
    uint16_t m_w;
    uint16_t m_h;
    int16_t m_x;
    int16_t m_y;
    uint32_t m_abstime;
    uint32_t m_reltime;
    uint8_t m_interleave;
    uint32_t m_bits;
};

#endif /* CHUNK_ANHD_HH */
