/* chunk_bmhd.hh
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CHUNK_BMHD_HH
#define CHUNK_BMHD_HH

#include "chunk_generic.hh"

class ChunkBMHD : public ChunkGeneric {
public:
    json_t *dump_as_json( const shared_properties_t &shared_props,
                          bool include_raw ) const override;

    static const uint8_t MASK_NONE = 0;
    static const uint8_t MASK_HAS_MASK = 1;
    static const uint8_t MASK_HAS_TRANSPARENCY_COLOR = 2;
    static const uint8_t MASK_LASSO = 3;

    static const uint8_t COMPRESSION_NONE = 0;
    static const uint8_t COMPRESSION_BYTERUN1 = 1;

    static const uint8_t CMAP_FLAG_8BIT = 1 << 7;

    uint16_t width() const;
    uint16_t height() const;
    uint8_t nplanes() const;
    uint8_t masking() const;
    uint8_t compression() const;
    uint8_t cmap_flags() const;
    uint16_t transparencyColor() const;

    size_t line_pitch() const;
protected:
    int parse_chunk( const std::vector< char > &data ) override;
private:
    uint16_t m_width = 0, m_height = 0;
    int16_t m_pos_x = 0, m_pos_y = 0;
    uint8_t m_no_of_planes = 0;
    uint8_t m_masking = 0;
    uint8_t m_compression = 0;
    uint8_t m_cmap_flags = 0;
    uint16_t m_transparency_color = 0;
    uint8_t m_xaspect = 0;
    uint8_t m_yaspect = 0;
    uint16_t m_page_width = 0;
    uint16_t m_page_height = 0;
};

#endif /* CHUNK_BMHD_HH */
