/* chunk_camg.cc
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "chunk_camg.hh"

int ChunkCAMG::parse_chunk( const std::vector< char > &data )
{
    auto chunklen = data.size();

    if ( m_chunk_type != CHUNKNAME( "CAMG" ) ) {
        return -EINVAL;
    }

    const char *buf = &data[0];

    if ( chunklen != 4 ) {
        LOG_DEBUG( "invalid CAMG chunk size: %u", chunklen );
        return -EINVAL;
    }

    m_display_mode = betoh_u32( get_u32( buf, 0 ) );

    if ( ( ! ( m_display_mode & MONITOR_ID_MASK ) ) ||
         ( ( m_display_mode & EXTENDED_MODE ) && ( ! ( m_display_mode & 0xFFFF0000 ) ) ) ) {
        m_display_mode &= ( ~ ( EXTENDED_MODE | SPRITES | GENLOCK_AUDIO | GENLOCK_VIDEO | VP_HIDE ) );
    }

    if ( ( m_display_mode & 0xFFFF0000 ) && ( ! ( m_display_mode & EXTENDED_MODE ) ) ) {
        m_display_mode = 0;
    }

    return 0;
}

json_t *ChunkCAMG::dump_as_json( const shared_properties_t &shared_props,
                                 bool include_raw ) const
{
    LOG_DEBUG( "called" );
    auto jroot = json_object();

    json_object_set_new( jroot, "id", json_string( "CAMG" ) );
    json_object_set_new( jroot, "displayMode", json_integer( m_display_mode ) );

    json_object_set_new( jroot, "extraHalfBrite", json_integer( !!( m_display_mode & EXTRA_HALFBRITE ) ) );
    json_object_set_new( jroot, "ham", json_integer( !!( m_display_mode & HAM ) ) );

    if ( include_raw ) {
        dump_raw_to_json( jroot );
    }

    return jroot;
}

uint32_t ChunkCAMG::display_mode() const
{
    return m_display_mode;
}
