/* chunk_camg.hh
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CHUNK_CAMG_HH
#define CHUNK_CAMG_HH

#include "chunk_generic.hh"

class ChunkCAMG : public ChunkGeneric {
public:
    json_t *dump_as_json( const shared_properties_t &shared_props,
                          bool include_raw ) const override;

    uint32_t display_mode() const;

    // graphics/view.h
    static const uint32_t GENLOCK_VIDEO = 0x0002;
    static const uint32_t LACE = 0x0004;
    static const uint32_t DOUBLESCAN = 0x0008;
    static const uint32_t SUPERHIRES = 0x0020;
    static const uint32_t PFBA = 0x0040;
    static const uint32_t EXTRA_HALFBRITE = 0x0080;
    static const uint32_t GENLOCK_AUDIO = 0x0100;
    static const uint32_t DUALPF = 0x0400;
    static const uint32_t HAM = 0x0800;
    static const uint32_t EXTENDED_MODE = 0x1000;
    static const uint32_t VP_HIDE = 0x2000;
    static const uint32_t SPRITES = 0x4000;
    static const uint32_t HIRES = 0x8000;

    // graphics/modeid.h
    static const uint32_t MONITOR_ID_MASK = 0xFFFF1000;

    static const uint32_t DEFAULT_MONITOR_ID = 0x00000000;
    static const uint32_t NTSC_MONITOR_ID = 0x00011000;
    static const uint32_t PAL_MONITOR_ID = 0x00021000;

    static const uint32_t LORES_KEY = 0x00000000;
    static const uint32_t HIRES_KEY = 0x00008000;
    static const uint32_t SUPER_KEY = 0x00008020;
    static const uint32_t HAM_KEY = 0x00000800;
    static const uint32_t LORESLACE_KEY = 0x00000004;
    static const uint32_t HIRESLACE_KEY = 0x00008004;
    static const uint32_t SUPERLACE_KEY = 0x00008024;
    static const uint32_t HAMLACE_KEY = 0x00000804;
    static const uint32_t LORESDPF_KEY = 0x00000400;
    static const uint32_t HIRESDPF_KEY = 0x00008400;
    static const uint32_t SUPERDPF_KEY = 0x00008420;
    static const uint32_t LORESLACEDPF_KEY = 0x00000404;
    static const uint32_t HIRESLACEDPF_KEY = 0x00008404;
    static const uint32_t SUPERLACEDPF_KEY = 0x00008424;
    static const uint32_t LORESDPF2_KEY = 0x00000440;
    static const uint32_t HIRESDPF2_KEY = 0x00008440;
    static const uint32_t SUPERDPF2_KEY = 0x00008460;
    static const uint32_t LORESLACEDPF2_KEY = 0x00000444;
    static const uint32_t HIRESLACEDPF2_KEY = 0x00008444;
    static const uint32_t SUPERLACEDPF2_KEY = 0x00008464;
    static const uint32_t EXTRAHALFBRITE_KEY = 0x00000080;
    static const uint32_t EXTRAHALFBRITELACE_KEY = 0x00000084;
    static const uint32_t HIRESHAM_KEY = 0x00008800;
    static const uint32_t SUPERHAM_KEY = 0x00008820;
    static const uint32_t HIRESEHB_KEY = 0x00008080;
    static const uint32_t SUPEREHB_KEY = 0x000080a0;
    static const uint32_t HIRESHAMLACE_KEY = 0x00008804;
    static const uint32_t SUPERHAMLACE_KEY = 0x00008824;
    static const uint32_t HIRESEHBLACE_KEY = 0x00008084;
    static const uint32_t SUPEREHBLACE_KEY = 0x000080a4;
    static const uint32_t LORESSDBL_KEY = 0x00000008;
    static const uint32_t LORESHAMSDBL_KEY = 0x00000808;
    static const uint32_t LORESEHBSDBL_KEY = 0x00000088;
    static const uint32_t HIRESHAMSDBL_KEY = 0x00008808;

    static const uint32_t VGA_MONITOR_ID = 0x00031000;
    static const uint32_t VGAEXTRALORES_KEY = 0x00031004;
    static const uint32_t VGALORES_KEY = 0x00039004;
    static const uint32_t VGAPRODUCT_KEY = 0x00039024;
    static const uint32_t VGAHAM_KEY = 0x00031804;
    static const uint32_t VGAEXTRALORESLACE_KEY = 0x00031005;
    static const uint32_t VGALORESLACE_KEY = 0x00039005;
    static const uint32_t VGAPRODUCTLACE_KEY = 0x00039025;
    static const uint32_t VGAHAMLACE_KEY = 0x00031805;
    static const uint32_t VGAEXTRALORESDPF_KEY = 0x00031404;
    static const uint32_t VGALORESDPF_KEY = 0x00039404;
    static const uint32_t VGAPRODUCTDPF_KEY = 0x00039424;
    static const uint32_t VGAEXTRALORESLACEDPF_KEY = 0x00031405;
    static const uint32_t VGALORESLACEDPF_KEY = 0x00039405;
    static const uint32_t VGAPRODUCTLACEDPF_KEY = 0x00039425;
    static const uint32_t VGAEXTRALORESDPF2_KEY = 0x00031444;
    static const uint32_t VGALORESDPF2_KEY = 0x00039444;
    static const uint32_t VGAPRODUCTDPF2_KEY = 0x00039464;
    static const uint32_t VGAEXTRALORESLACEDPF2_KEY = 0x00031445;
    static const uint32_t VGALORESLACEDPF2_KEY = 0x00039445;
    static const uint32_t VGAPRODUCTLACEDPF2_KEY = 0x00039465;
    static const uint32_t VGAEXTRAHALFBRITE_KEY = 0x00031084;
    static const uint32_t VGAEXTRAHALFBRITELACE_KEY = 0x00031085;
    static const uint32_t VGAPRODUCTHAM_KEY = 0x00039824;
    static const uint32_t VGALORESHAM_KEY = 0x00039804;
    static const uint32_t VGAEXTRALORESHAM_KEY = VGAHAM_KEY;
    static const uint32_t VGAPRODUCTHAMLACE_KEY = 0x00039825;
    static const uint32_t VGALORESHAMLACE_KEY = 0x00039805;
    static const uint32_t VGAEXTRALORESHAMLACE_KEY = VGAHAMLACE_KEY;
    static const uint32_t VGAEXTRALORESEHB_KEY = VGAEXTRAHALFBRITE_KEY;
    static const uint32_t VGAEXTRALORESEHBLACE_KEY = VGAEXTRAHALFBRITELACE_KEY;
    static const uint32_t VGALORESEHB_KEY = 0x00039084;
    static const uint32_t VGALORESEHBLACE_KEY = 0x00039085;
    static const uint32_t VGAEHB_KEY = 0x000390a4;
    static const uint32_t VGAEHBLACE_KEY = 0x000390a5;
    static const uint32_t VGAEXTRALORESDBL_KEY = 0x00031000;
    static const uint32_t VGALORESDBL_KEY = 0x00039000;
    static const uint32_t VGAPRODUCTDBL_KEY = 0x00039020;
    static const uint32_t VGAEXTRALORESHAMDBL_KEY = 0x00031800;
    static const uint32_t VGALORESHAMDBL_KEY = 0x00039800;
    static const uint32_t VGAPRODUCTHAMDBL_KEY = 0x00039820;
    static const uint32_t VGAEXTRALORESEHBDBL_KEY = 0x00031080;
    static const uint32_t VGALORESEHBDBL_KEY = 0x00039080;
    static const uint32_t VGAPRODUCTEHBDBL_KEY = 0x000390a0;

    static const uint32_t A2024_MONITOR_ID = 0x00041000;
    static const uint32_t A2024TENHERTZ_KEY = 0x00041000;
    static const uint32_t A2024FIFTEENHERTZ_KEY = 0x00049000;

    static const uint32_t PROTO_MONITOR_ID = 0x00051000;

    static const uint32_t EURO72_MONITOR_ID = 0x00061000;
    static const uint32_t EURO72EXTRALORES_KEY = 0x00061004;
    static const uint32_t EURO72LORES_KEY = 0x00069004;
    static const uint32_t EURO72PRODUCT_KEY = 0x00069024;
    static const uint32_t EURO72HAM_KEY = 0x00061804;
    static const uint32_t EURO72EXTRALORESLACE_KEY = 0x00061005;
    static const uint32_t EURO72LORESLACE_KEY = 0x00069005;
    static const uint32_t EURO72PRODUCTLACE_KEY = 0x00069025;
    static const uint32_t EURO72HAMLACE_KEY = 0x00061805;
    static const uint32_t EURO72EXTRALORESDPF_KEY = 0x00061404;
    static const uint32_t EURO72LORESDPF_KEY = 0x00069404;
    static const uint32_t EURO72PRODUCTDPF_KEY = 0x00069424;
    static const uint32_t EURO72EXTRALORESLACEDPF_KEY = 0x00061405;
    static const uint32_t EURO72LORESLACEDPF_KEY = 0x00069405;
    static const uint32_t EURO72PRODUCTLACEDPF_KEY = 0x00069425;
    static const uint32_t EURO72EXTRALORESDPF2_KEY = 0x00061444;
    static const uint32_t EURO72LORESDPF2_KEY = 0x00069444;
    static const uint32_t EURO72PRODUCTDPF2_KEY = 0x00069464;
    static const uint32_t EURO72EXTRALORESLACEDPF2_KEY = 0x00061445;
    static const uint32_t EURO72LORESLACEDPF2_KEY = 0x00069445;
    static const uint32_t EURO72PRODUCTLACEDPF2_KEY = 0x00069465;
    static const uint32_t EURO72EXTRAHALFBRITE_KEY = 0x00061084;
    static const uint32_t EURO72EXTRAHALFBRITELACE_KEY = 0x00061085;
    static const uint32_t EURO72PRODUCTHAM_KEY = 0x00069824;
    static const uint32_t EURO72PRODUCTHAMLACE_KEY = 0x00069825;
    static const uint32_t EURO72LORESHAM_KEY = 0x00069804;
    static const uint32_t EURO72LORESHAMLACE_KEY = 0x00069805;
    static const uint32_t EURO72EXTRALORESHAM_KEY = EURO72HAM_KEY;
    static const uint32_t EURO72EXTRALORESHAMLACE_KEY = EURO72HAMLACE_KEY;
    static const uint32_t EURO72EXTRALORESEHB_KEY = EURO72EXTRAHALFBRITE_KEY;
    static const uint32_t EURO72EXTRALORESEHBLACE_KEY = EURO72EXTRAHALFBRITELACE_KEY;
    static const uint32_t EURO72LORESEHB_KEY = 0x00069084;
    static const uint32_t EURO72LORESEHBLACE_KEY = 0x00069085;
    static const uint32_t EURO72EHB_KEY = 0x000690a4;
    static const uint32_t EURO72EHBLACE_KEY = 0x000690a5;
    static const uint32_t EURO72EXTRALORESDBL_KEY = 0x00061000;
    static const uint32_t EURO72LORESDBL_KEY = 0x00069000;
    static const uint32_t EURO72PRODUCTDBL_KEY = 0x00069020;
    static const uint32_t EURO72EXTRALORESHAMDBL_KEY = 0x00061800;
    static const uint32_t EURO72LORESHAMDBL_KEY = 0x00069800;
    static const uint32_t EURO72PRODUCTHAMDBL_KEY = 0x00069820;
    static const uint32_t EURO72EXTRALORESEHBDBL_KEY = 0x00061080;
    static const uint32_t EURO72LORESEHBDBL_KEY = 0x00069080;
    static const uint32_t EURO72PRODUCTEHBDBL_KEY = 0x000690a0;

    static const uint32_t EURO36_MONITOR_ID = 0x00071000;

    static const uint32_t SUPER72_MONITOR_ID = 0x00081000;
    static const uint32_t SUPER72LORESDBL_KEY = 0x00081008;
    static const uint32_t SUPER72HIRESDBL_KEY = 0x00089008;
    static const uint32_t SUPER72SUPERDBL_KEY = 0x00089028;
    static const uint32_t SUPER72LORESHAMDBL_KEY = 0x00081808;
    static const uint32_t SUPER72HIRESHAMDBL_KEY = 0x00089808;
    static const uint32_t SUPER72SUPERHAMDBL_KEY = 0x00089828;
    static const uint32_t SUPER72LORESEHBDBL_KEY = 0x00081088;
    static const uint32_t SUPER72HIRESEHBDBL_KEY = 0x00089088;
    static const uint32_t SUPER72SUPEREHBDBL_KEY = 0x000890a8;

    static const uint32_t DBLNTSC_MONITOR_ID = 0x00091000;
    static const uint32_t DBLNTSCLORES_KEY = 0x00091000;
    static const uint32_t DBLNTSCLORESFF_KEY = 0x00091004;
    static const uint32_t DBLNTSCLORESHAM_KEY = 0x00091800;
    static const uint32_t DBLNTSCLORESHAMFF_KEY = 0x00091804;
    static const uint32_t DBLNTSCLORESEHB_KEY = 0x00091080;
    static const uint32_t DBLNTSCLORESEHBFF_KEY = 0x00091084;
    static const uint32_t DBLNTSCLORESLACE_KEY = 0x00091005;
    static const uint32_t DBLNTSCLORESHAMLACE_KEY = 0x00091805;
    static const uint32_t DBLNTSCLORESEHBLACE_KEY = 0x00091085;
    static const uint32_t DBLNTSCLORESDPF_KEY = 0x00091400;
    static const uint32_t DBLNTSCLORESDPFFF_KEY = 0x00091404;
    static const uint32_t DBLNTSCLORESDPFLACE_KEY = 0x00091405;
    static const uint32_t DBLNTSCLORESDPF2_KEY = 0x00091440;
    static const uint32_t DBLNTSCLORESDPF2FF_KEY = 0x00091444;
    static const uint32_t DBLNTSCLORESDPF2LACE_KEY = 0x00091445;
    static const uint32_t DBLNTSCHIRES_KEY = 0x00099000;
    static const uint32_t DBLNTSCHIRESFF_KEY = 0x00099004;
    static const uint32_t DBLNTSCHIRESHAM_KEY = 0x00099800;
    static const uint32_t DBLNTSCHIRESHAMFF_KEY = 0x00099804;
    static const uint32_t DBLNTSCHIRESLACE_KEY = 0x00099005;
    static const uint32_t DBLNTSCHIRESHAMLACE_KEY = 0x00099805;
    static const uint32_t DBLNTSCHIRESEHB_KEY = 0x00099080;
    static const uint32_t DBLNTSCHIRESEHBFF_KEY = 0x00099084;
    static const uint32_t DBLNTSCHIRESEHBLACE_KEY = 0x00099085;
    static const uint32_t DBLNTSCHIRESDPF_KEY = 0x00099400;
    static const uint32_t DBLNTSCHIRESDPFFF_KEY = 0x00099404;
    static const uint32_t DBLNTSCHIRESDPFLACE_KEY = 0x00099405;
    static const uint32_t DBLNTSCHIRESDPF2_KEY = 0x00099440;
    static const uint32_t DBLNTSCHIRESDPF2FF_KEY = 0x00099444;
    static const uint32_t DBLNTSCHIRESDPF2LACE_KEY = 0x00099445;
    static const uint32_t DBLNTSCEXTRALORES_KEY = 0x00091200;
    static const uint32_t DBLNTSCEXTRALORESHAM_KEY = 0x00091a00;
    static const uint32_t DBLNTSCEXTRALORESEHB_KEY = 0x00091280;
    static const uint32_t DBLNTSCEXTRALORESDPF_KEY = 0x00091600;
    static const uint32_t DBLNTSCEXTRALORESDPF2_KEY = 0x00091640;
    static const uint32_t DBLNTSCEXTRALORESFF_KEY = 0x00091204;
    static const uint32_t DBLNTSCEXTRALORESHAMFF_KEY = 0x00091a04;
    static const uint32_t DBLNTSCEXTRALORESEHBFF_KEY = 0x00091284;
    static const uint32_t DBLNTSCEXTRALORESDPFFF_KEY = 0x00091604;
    static const uint32_t DBLNTSCEXTRALORESDPF2FF_KEY = 0x00091644;
    static const uint32_t DBLNTSCEXTRALORESLACE_KEY = 0x00091205;
    static const uint32_t DBLNTSCEXTRALORESHAMLACE_KEY = 0x00091a05;
    static const uint32_t DBLNTSCEXTRALORESEHBLACE_KEY = 0x00091285;
    static const uint32_t DBLNTSCEXTRALORESDPFLACE_KEY = 0x00091605;
    static const uint32_t DBLNTSCEXTRALORESDPF2LACE_KEY = 0x00091645;

    static const uint32_t DBLPAL_MONITOR_ID = 0x000a1000;
    static const uint32_t DBLPALLORES_KEY = 0x000a1000;
    static const uint32_t DBLPALLORESFF_KEY = 0x000a1004;
    static const uint32_t DBLPALLORESHAM_KEY = 0x000a1800;
    static const uint32_t DBLPALLORESHAMFF_KEY = 0x000a1804;
    static const uint32_t DBLPALLORESEHB_KEY = 0x000a1080;
    static const uint32_t DBLPALLORESEHBFF_KEY = 0x000a1084;
    static const uint32_t DBLPALLORESLACE_KEY = 0x000a1005;
    static const uint32_t DBLPALLORESHAMLACE_KEY = 0x000a1805;
    static const uint32_t DBLPALLORESEHBLACE_KEY = 0x000a1085;
    static const uint32_t DBLPALLORESDPF_KEY = 0x000a1400;
    static const uint32_t DBLPALLORESDPFFF_KEY = 0x000a1404;
    static const uint32_t DBLPALLORESDPFLACE_KEY = 0x000a1405;
    static const uint32_t DBLPALLORESDPF2_KEY = 0x000a1440;
    static const uint32_t DBLPALLORESDPF2FF_KEY = 0x000a1444;
    static const uint32_t DBLPALLORESDPF2LACE_KEY = 0x000a1445;
    static const uint32_t DBLPALHIRES_KEY = 0x000a9000;
    static const uint32_t DBLPALHIRESFF_KEY = 0x000a9004;
    static const uint32_t DBLPALHIRESHAM_KEY = 0x000a9800;
    static const uint32_t DBLPALHIRESHAMFF_KEY = 0x000a9804;
    static const uint32_t DBLPALHIRESLACE_KEY = 0x000a9005;
    static const uint32_t DBLPALHIRESHAMLACE_KEY = 0x000a9805;
    static const uint32_t DBLPALHIRESEHB_KEY = 0x000a9080;
    static const uint32_t DBLPALHIRESEHBFF_KEY = 0x000a9084;
    static const uint32_t DBLPALHIRESEHBLACE_KEY = 0x000a9085;
    static const uint32_t DBLPALHIRESDPF_KEY = 0x000a9400;
    static const uint32_t DBLPALHIRESDPFFF_KEY = 0x000a9404;
    static const uint32_t DBLPALHIRESDPFLACE_KEY = 0x000a9405;
    static const uint32_t DBLPALHIRESDPF2_KEY = 0x000a9440;
    static const uint32_t DBLPALHIRESDPF2FF_KEY = 0x000a9444;
    static const uint32_t DBLPALHIRESDPF2LACE_KEY = 0x000a9445;
    static const uint32_t DBLPALEXTRALORES_KEY = 0x000a1200;
    static const uint32_t DBLPALEXTRALORESHAM_KEY = 0x000a1a00;
    static const uint32_t DBLPALEXTRALORESEHB_KEY = 0x000a1280;
    static const uint32_t DBLPALEXTRALORESDPF_KEY = 0x000a1600;
    static const uint32_t DBLPALEXTRALORESDPF2_KEY = 0x000a1640;
    static const uint32_t DBLPALEXTRALORESFF_KEY = 0x000a1204;
    static const uint32_t DBLPALEXTRALORESHAMFF_KEY = 0x000a1a04;
    static const uint32_t DBLPALEXTRALORESEHBFF_KEY = 0x000a1284;
    static const uint32_t DBLPALEXTRALORESDPFFF_KEY = 0x000a1604;
    static const uint32_t DBLPALEXTRALORESDPF2FF_KEY = 0x000a1644;
    static const uint32_t DBLPALEXTRALORESLACE_KEY = 0x000a1205;
    static const uint32_t DBLPALEXTRALORESHAMLACE_KEY = 0x000a1a05;
    static const uint32_t DBLPALEXTRALORESEHBLACE_KEY = 0x000a1285;
    static const uint32_t DBLPALEXTRALORESDPFLACE_KEY = 0x000a1605;
    static const uint32_t DBLPALEXTRALORESDPF2LACE_KEY = 0x000a1645;
protected:
    int parse_chunk( const std::vector< char > &data ) override;
private:
    uint32_t m_display_mode = 0;
};

#endif /* CHUNK_CAMG_HH */
