/* chunk_chrs.hh
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CHUNK_CHRS_HH
#define CHUNK_CHRS_HH

#include "chunk_generic.hh"

class ChunkCHRS : public ChunkGeneric {
public:
    std::string dump_as_readable_text( const shared_properties_t &shared_props ) override;
    json_t *dump_as_json( const shared_properties_t &shared_props,
                          bool include_raw ) const override;
    int dump_as_raw( const shared_properties_t &shared_props,
                     std::vector< char > &data ) const override;

    virtual std::string text() const;
    virtual const std::vector< char > raw_text() const;
    virtual std::string parsed_text_utf8() const;
protected:
    int parse_chunk( const std::vector< char > &data ) override;
    static int convert_control_sequences( const std::string &in,
                                          std::string &out );
    static int convert_to_utf8( const std::string &in,
                                std::string &out );
    std::vector< char > m_raw_text;
};

#endif /* CHUNK_CHRS_HH */
