/* chunk_cstr.cc
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "chunk_cstr.hh"
#include <algorithm>
#include <iconv.h>
#include <cstring>

int ChunkCSTR::parse_chunk( const std::vector< char > &data )
{
    auto chunklen = data.size();
    const char *buf = &data[0];

    if ( m_chunk_type != CHUNKNAME( "CSTR" ) ) {
        return -EINVAL;
    }

    m_raw_text.resize( chunklen );
    std::copy_n( buf, chunklen,
                 m_raw_text.begin() );

    LOG_DEBUG( "Parsed CSTR" );

    return 0;
}

std::string ChunkCSTR::text() const
{
    auto res = ChunkCHRS::text();

    if ( res.size() > 0 ) {
        if ( res[ res.size() - 1 ] == '\0' ) {
            res.resize( res.size() - 1 );
        } else {
            LOG_VERBOSE( "null byte missing in CSTR chunk" );
        }
    }

    return res;
}
