/* chunk_dchg.cc
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "chunk_dchg.hh"

int ChunkDCHG::parse_chunk( const std::vector< char > &data )
{
    auto chunklen = data.size();

    LOG_DEBUG( "called" );

    if ( m_chunk_type != CHUNKNAME( "DCHG" ) ) {
        return -EINVAL;
    }

    const char *buf = &data[0];

    if ( chunklen != 4 ) {
        LOG_DEBUG( "invalid DCHG chunk size: %u", chunklen );
        return -EINVAL;
    }

    m_frame_rate = (int32_t)betoh_u32( get_u32( buf, 0 ) );

    return 0;
}

json_t *ChunkDCHG::dump_as_json( const shared_properties_t &shared_props,
                                 bool include_raw ) const
{
    LOG_DEBUG( "called" );
    auto jroot = json_object();

    json_object_set_new( jroot, "id", json_string( CHUNKTYPE2NAME( m_chunk_type ).c_str() ) );
    json_object_set_new( jroot, "frameRateMs", json_integer( m_frame_rate ) );

    if ( include_raw ) {
        dump_raw_to_json( jroot );
    }

    return jroot;
}
