/* chunk_dest.cc
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "chunk_dest.hh"

int ChunkDEST::parse_chunk( const std::vector< char > &data )
{
    auto chunklen = data.size();

    if ( m_chunk_type != CHUNKNAME( "DEST" ) ) {
        return -EINVAL;
    }

    const char *buf = &data[0];

    if ( chunklen != 8 ) {
        LOG_DEBUG( "invalid DEST chunk size: %u", chunklen );
        return -EINVAL;
    }

    m_depth = get_u8( buf, 0 );
    m_plane_pick = betoh_u16( get_u16( buf, 2 ) );
    m_plane_on_off = betoh_u16( get_u16( buf, 4 ) );
    m_plane_mask = betoh_u16( get_u16( buf, 6 ) );

    return 0;
}

json_t *ChunkDEST::dump_as_json( const shared_properties_t &shared_props,
                                 bool include_raw ) const
{
    LOG_DEBUG( "called" );
    auto jroot = json_object();

    json_object_set_new( jroot, "id", json_string( "DEST" ) );
    json_object_set_new( jroot, "destinationDepth", json_integer( m_depth ) );
    json_object_set_new( jroot, "planePick", json_integer( m_plane_pick ) );
    json_object_set_new( jroot, "planeOnOff", json_integer( m_plane_on_off ) );
    json_object_set_new( jroot, "planeMask", json_integer( m_plane_mask ) );

    if ( include_raw ) {
        dump_raw_to_json( jroot );
    }

    return jroot;
}
