/* chunk_dgbl.hh
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CHUNK_DGBL_HH
#define CHUNK_DGBL_HH

#include "chunk_generic.hh"

class ChunkDGBL : public ChunkGeneric {
public:
    json_t *dump_as_json( const shared_properties_t &shared_props,
                          bool include_raw ) const override;
    int fill_info( IFFInfo &info ) const override;

    static const uint8_t COMPRESSION_NONE = 0;
    static const uint8_t COMPRESSION_RLE = 1;
    static const uint8_t COMPRESSION_HUFFMAN = 2;
    static const uint8_t COMPRESSION_DYNAMIC_HUFFMAN = 3;
    static const uint8_t COMPRESSION_JPEG = 4;

    uint16_t display_width() const;
    uint16_t display_height() const;
    uint16_t compression() const;
protected:
    int parse_chunk( const std::vector< char > &data ) override;
private:
    uint16_t m_display_width = 0, m_display_height = 0;
    uint16_t m_compression = 0;
    uint8_t m_xaspect = 0;
    uint8_t m_yaspect = 0;
};

#endif /* CHUNK_DGBL_HH */
