/* chunk_dloc.cc
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "chunk_dloc.hh"

int ChunkDLOC::parse_chunk( const std::vector< char > &data )
{
    auto chunklen = data.size();

    LOG_DEBUG( "called" );

    if ( m_chunk_type != CHUNKNAME( "DLOC" ) ) {
        return -EINVAL;
    }

    const char *buf = &data[0];

    if ( chunklen != 8 ) {
        LOG_DEBUG( "invalid DLOC chunk size: %u", chunklen );
        return -EINVAL;
    }

    m_width = betoh_u16( get_u16( buf, 0 ) );
    m_height = betoh_u16( get_u16( buf, 2 ) );
    m_x = (int16_t)betoh_u16( get_u16( buf, 4 ) );
    m_y = (int16_t)betoh_u16( get_u16( buf, 6 ) );

    return 0;
}

json_t *ChunkDLOC::dump_as_json( const shared_properties_t &shared_props,
                                 bool include_raw ) const
{
    LOG_DEBUG( "called" );
    auto jroot = json_object();

    json_object_set_new( jroot, "id", json_string( CHUNKTYPE2NAME( m_chunk_type ).c_str() ) );
    json_object_set_new( jroot, "width", json_integer( m_width ) );
    json_object_set_new( jroot, "height", json_integer( m_height ) );
    json_object_set_new( jroot, "x", json_integer( m_x ) );
    json_object_set_new( jroot, "y", json_integer( m_y ) );

    if ( include_raw ) {
        dump_raw_to_json( jroot );
    }

    return jroot;
}

uint16_t ChunkDLOC::width() const
{
    return m_width;
}

uint16_t ChunkDLOC::height() const
{
    return m_height;
}

int16_t ChunkDLOC::x() const
{
    return m_x;
}

int16_t ChunkDLOC::y() const
{
    return m_y;
}
