/* chunk_dpel.hh
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CHUNK_DPEL_HH
#define CHUNK_DPEL_HH

#include "chunk_generic.hh"

class ChunkDPEL : public ChunkGeneric {
public:
    json_t *dump_as_json( const shared_properties_t &shared_props,
                          bool include_raw ) const override;

    static const uint8_t TYPE_RED = 1;
    static const uint8_t TYPE_GREEN = 2;
    static const uint8_t TYPE_BLUE = 3;
    static const uint8_t TYPE_ALPHA = 4;
    static const uint8_t TYPE_YELLOW = 5;
    static const uint8_t TYPE_CYAN = 6;
    static const uint8_t TYPE_MAGENTA = 7;
    static const uint8_t TYPE_BLACK = 8;
    static const uint8_t TYPE_MASK = 9;
    static const uint8_t TYPE_ZBUFFER = 10;
    static const uint8_t TYPE_OPACITY = 11;
    static const uint8_t TYPE_LINEARKEY = 12;
    static const uint8_t TYPE_BINARYKEY = 13;

    class pixel_depth {
    public:
        uint16_t type = 0;
        uint16_t depth = 0;
    };

    std::vector< pixel_depth > elements() const;
protected:
    int parse_chunk( const std::vector< char > &data ) override;
private:
    std::vector< pixel_depth > m_elements;
};

#endif /* CHUNK_DPEL_HH */
