/* chunk_fons.cc
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "chunk_fons.hh"

int ChunkFONS::parse_chunk( const std::vector< char > &data )
{
    auto chunklen = data.size();

    if ( m_chunk_type != CHUNKNAME( "FONS" ) ) {
        return -EINVAL;
    }

    const char *buf = &data[0];

    if ( chunklen < 6 ) {
        return -EINVAL;
    }

    m_font_id = get_u8( buf, 0 );
    m_proportional = get_u8( buf, 2 );
    m_serif = get_u8( buf, 3 );
    m_font_name = std::string( buf + 4, chunklen - 4 - 1 );  // 4 for the header and 1 for the final null byte

    return 0;
}

json_t *ChunkFONS::dump_as_json( const shared_properties_t &shared_props,
                                 bool include_raw ) const
{
    LOG_DEBUG( "called" );

    auto jroot = json_object();

    json_object_set_new( jroot, "id", json_string( "FONS" ) );
    json_object_set_new( jroot, "fontID", json_integer( m_font_id ) );

    json_object_set_new( jroot, "proportionalValue", json_integer( m_proportional ) );
    switch ( m_proportional ) {
        case PROPORTIONAL_UNKNOWN:
            json_object_set_new( jroot, "proportional", json_string( "unknown" ) );
            break;
        case PROPORTIONAL_NO:
            json_object_set_new( jroot, "proportional", json_string( "no" ) );
            break;
        case PROPORTIONAL_YES:
            json_object_set_new( jroot, "proportional", json_string( "yes" ) );
            break;
        default:
            json_object_set_new( jroot, "proportional", json_string( "invalid" ) );
            break;
    }

    json_object_set_new( jroot, "serifValue", json_integer( m_serif ) );
    switch ( m_serif ) {
        case SERIF_UNKNOWN:
            json_object_set_new( jroot, "serif", json_string( "unknown" ) );
            break;
        case SERIF_NO:
            json_object_set_new( jroot, "serif", json_string( "no" ) );
            break;
        case SERIF_YES:
            json_object_set_new( jroot, "serif", json_string( "yes" ) );
            break;
        default:
            json_object_set_new( jroot, "serif", json_string( "invalid" ) );
            break;
    }

    json_object_set_new( jroot, "fontName", json_string( m_font_name.c_str() ) );

    return jroot;
}
