/* chunk_list.hh
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CHUNK_LIST_HH
#define CHUNK_LIST_HH

#include "chunk_generic.hh"
#include "chunk_prop.hh"

class ChunkLIST : public ChunkGeneric {
public:
    std::string dump_as_readable_text( const shared_properties_t &shared_props ) override;
    json_t *dump_as_json( const shared_properties_t &shared_props,
                          bool include_raw ) const override;
    int dump_as_binary( const shared_properties_t &shared_props,
                        std::list< IFFBinaryResult > &output,
                        bool include_raw ) const;
    int fill_info( IFFInfo &info ) const override;
protected:
    int parse_chunk( const std::vector< char > &data ) override;
private:
    shared_properties_t add_prop_chunks( const shared_properties_t &shared_props ) const;

    uint32_t m_list_type = 0;
    std::list< std::shared_ptr< ChunkPROP > > m_prop_chunks;
    std::list< std::shared_ptr< ChunkGeneric > > m_other_chunks;
};

#endif /* CHUNK_LIST_HH */
