/* chunk_prop.cc
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "chunk_prop.hh"

int ChunkPROP::parse_chunk( const std::vector< char > &data )
{
    auto chunklen = data.size();

    if ( chunklen < 12 ) {
        return -EINVAL;
    }

    if ( m_chunk_type != CHUNKNAME( "PROP" ) ) {
        return -EINVAL;
    }

    LOG_DEBUG( "found PROP chunk" );
    LOG_DEBUG( "parse %u bytes", chunklen );

    const char *buf = &data[0];

    m_content_hint = betoh_u32( get_u32( buf, 0 ) );

    buf += 4;
    chunklen -= 4;

    while ( chunklen >= 8 ) {
        //auto inner_chunktype = betoh_u32( get_u32( buf, 0 ) );
        auto inner_chunklen = betoh_u32( get_u32( buf, 4 ) );

        if ( inner_chunklen + 8 > chunklen ) {
            LOG_DEBUG( "Invalid chunk length" );
            return -EINVAL;
        }

        size_t chunk_parsed_len;

        auto chunk = std::make_shared< ChunkGeneric >();
        if ( chunk->parse( buf, chunklen, chunk_parsed_len ) != 0 ) {
            LOG_DEBUG( "Unsupported chunk %.*s", 4, buf );
            if ( fail_on_unsupported() ) {
                return -EINVAL;
            }
        }

        m_entries.push_back( chunk );

        if ( inner_chunklen & 1 ) {
            buf += 8 + inner_chunklen + 1;
            chunklen -= 8 + inner_chunklen + 1;
        } else {
            buf += 8 + inner_chunklen;
            chunklen -= 8 + inner_chunklen;
        }
    }

    return 0;
}

void ChunkPROP::extend_shared_props( shared_properties_t &props, const std::string &form_type )
{
    std::string prop_form_type = CHUNKTYPE2NAME( m_content_hint );

    for ( auto &chunk : m_entries ) {
        auto key = std::make_pair( prop_form_type, chunk->getChunkName() );

        LOG_DEBUG( "add property for key %s/%s", key.first.c_str(), key.second.c_str() );

        props[ key ] = chunk;
    }
}
