/* chunk_sprt.cc
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "chunk_sprt.hh"

int ChunkSPRT::parse_chunk( const std::vector< char > &data )
{
    auto chunklen = data.size();

    if ( m_chunk_type != CHUNKNAME( "SPRT" ) ) {
        return -EINVAL;
    }

    const char *buf = &data[0];

    if ( chunklen != 2 ) {
        LOG_DEBUG( "invalid SPRT chunk size: %u", chunklen );
        return -EINVAL;
    }

    m_sprite_precedence = betoh_u16( get_u16( buf, 0 ) );

    return 0;
}

json_t *ChunkSPRT::dump_as_json( const shared_properties_t &shared_props,
                                 bool include_raw ) const
{
    LOG_DEBUG( "called" );
    auto jroot = json_object();

    json_object_set_new( jroot, "id", json_string( "SPRT" ) );
    json_object_set_new( jroot, "spritePrecedence", json_integer( m_sprite_precedence ) );

    if ( include_raw ) {
        dump_raw_to_json( jroot );
    }

    return jroot;
}
