/* chunk_vhdr.hh
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CHUNK_VHDR_HH
#define CHUNK_VHDR_HH

#include "chunk_generic.hh"

class ChunkVHDR : public ChunkGeneric {
public:
    json_t *dump_as_json( const shared_properties_t &shared_props,
                          bool include_raw ) const override;
    int fill_info( IFFInfo &info ) const override;

    static const uint8_t COMPRESSION_NONE = 0;
    static const uint8_t COMPRESSION_FDE = 1;

    uint32_t one_shot_samples() const;
    uint32_t repeat_samples() const;
    uint8_t no_of_octaves() const;
    uint8_t compression() const;
    uint16_t samples_per_second() const;
    int32_t volume() const;
protected:
    int parse_chunk( const std::vector< char > &data ) override;
private:
    uint32_t m_one_shot_samples = 0;
    uint32_t m_repeat_samples = 0;
    uint32_t m_samples_per_cycle = 0;
    uint16_t m_samples_per_second = 0;
    uint8_t m_no_of_octaves = 0;
    uint8_t m_compression = 0;
    int32_t m_volume = 0;
};

#endif /* CHUNK_VHDR_HH */
