/* lowlevel_types.cc
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "lowlevel_types.hh"
#include <endian.h>

uint8_t get_u8( const void *ptr, size_t offset )
{
    const uint8_t *u8ptr = (const uint8_t*)ptr;
    u8ptr += offset;
    return *((const uint8_t*)u8ptr);
}

uint16_t get_u16( const void *ptr, size_t offset )
{
    const uint8_t *u8ptr = (const uint8_t*)ptr;
    u8ptr += offset;
    return *((const uint16_t*)u8ptr);
}

uint32_t get_u32( const void *ptr, size_t offset )
{
    const uint8_t *u8ptr = (const uint8_t*)ptr;
    u8ptr += offset;
    return *((const uint32_t*)u8ptr);
}

uint64_t get_u64( const void *ptr, size_t offset )
{
    const uint8_t *u8ptr = (const uint8_t*)ptr;
    u8ptr += offset;
    return *((const uint64_t*)u8ptr);
}

int8_t get_s8( const void *ptr, size_t offset )
{
    const uint8_t *u8ptr = (const uint8_t*)ptr;
    u8ptr += offset;
    return *((const int8_t*)u8ptr);
}

int16_t get_s16( const void *ptr, size_t offset )
{
    const uint8_t *u8ptr = (const uint8_t*)ptr;
    u8ptr += offset;
    return *((const int16_t*)u8ptr);
}

int32_t get_s32( const void *ptr, size_t offset )
{
    const uint8_t *u8ptr = (const uint8_t*)ptr;
    u8ptr += offset;
    return *((const int32_t*)u8ptr);
}

int64_t get_s64( const void *ptr, size_t offset )
{
    const uint8_t *u8ptr = (const uint8_t*)ptr;
    u8ptr += offset;
    return *((const int64_t*)u8ptr);
}

uint16_t betoh_u16( uint16_t v )
{
    return be16toh( v );
}

uint32_t betoh_u32( uint32_t v )
{
    return be32toh( v );
}

uint64_t betoh_u64( uint64_t v )
{
    return be32toh( v );
}

uint16_t htobe_u16( uint16_t v )
{
    return htobe16( v );
}

uint32_t htobe_u32( uint32_t v )
{
    return htobe32( v );
}

uint64_t htobe_u64( uint64_t v )
{
    return htobe32( v );
}
