/* util.cc
 * Copyright (C) 2020 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "util.hh"
#include <stdarg.h>

std::string formatStringToString( const char *format, ... )
{
    va_list va;
    int n, size = 1024;
    char *tstr;
    std::string string1;
        
    if ( format != NULL ) {
        // first create str
        while ( 1 ) {
            tstr = (char*)malloc( size );
            va_start( va, format );
            n = vsnprintf( tstr, size, format, va );
            va_end( va );
            if ( n > -1 && n < size ) break;
            free( tstr );
            size *= 2;
        }
        string1 = tstr;
        free( tstr );
        return string1;
    }
    return "";
}

void split_string( std::vector< std::string > &v,
                   const std::string &str1,
                   const char split_element )
{
    std::string::size_type start_pos = 0;

    if ( start_pos >= str1.length() ) {
        // empty string
        v.push_back( str1 );
        return;
    }

    std::string::size_type next_pos = str1.find( split_element, start_pos );

    if ( next_pos == std::string::npos ) {
        // no split_element found
        v.push_back( str1 );
        return;
    }

    // now there is at least one split element

    while ( start_pos >= 0 && start_pos < str1.length() ) {
        next_pos = str1.find( split_element, start_pos );

        int basename_length;

        if ( next_pos == std::string::npos ) {
            basename_length = str1.length() - start_pos;
        } else {
            basename_length = next_pos - start_pos;
        }

        std::string basename( str1, start_pos, basename_length );

        v.push_back( basename );

        if ( next_pos == std::string::npos ) {
            break;
        }

        start_pos = next_pos + 1;
    }

    if ( start_pos >= str1.length() ) {
        // ended with split_element so push empty string
        v.push_back( "" );
    }
}
